import express from 'express';
import Sequelize, { Transaction } from 'sequelize';
import { Sequelize as SequelizeTS } from 'sequelize-typescript';
/**
 * AOP 기능을 지원 하기 위해서 별도로 정의된 데이터베이스 처리 함수.
 * 주로 express router 가 timeout 으로 종료 되었을 때, 더 이상 실행 되지 않게 하기 위해서 사용 된다.
 * AOP 기능을 지원 하는 함수는 함수 이름의 제일 뒤에 '_AOP' 를 붙여야 한다. 함수명 제일 뒤에 붙는 '_AOP' 값으로 AOP 지원이 설정 된다.
 * AOP 함수는 파라미터로 첫번째는 req: express.Request, 두번째는 res: express.Response 를 받아야 한다.
 */
declare class DatabaseWebService {
    /**
     * id 로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param id
     * @param attributes
     * @param transaction
     * @throws CkError
     */
    static getDataOneById_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, id: number | undefined, attributes?: Array<string>, transaction?: Transaction): Promise<T | null>;
    /**
     * id(srl) 목록으로 테이블에서 데이터를 가져 온다.
     * return 값이 empty array 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * empty array, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param ids
     * @param attributes
     * @param transaction
     * @param order
     * @throws CkError
     */
    static getDatasByIds_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, ids: Array<number>, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<Array<T>>;
    /**
     * getDatasByIds 와 같은 역할을 하지만, 데이터가 아니라 데이터의 카운트를 구한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param ids
     * @param transaction
     * @throws CkError
     */
    static countDatasByIds_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, ids: Array<number>, transaction?: Transaction): Promise<number>;
    /**
     * rid 로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param rid
     * @param attributes
     * @param transaction
     * @throws CkError
     */
    static getDataOneByRid_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, rid: string | undefined, attributes?: Array<string>, transaction?: Transaction): Promise<T | null>;
    /**
     * rid 목록으로 테이블에서 데이터를 가져 온다.
     * return 값이 empty array 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * empty array, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param rids
     * @param attributes
     * @param transaction
     * @param order
     * @throws CkError
     */
    static getDatasByRids_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, rids: Array<string>, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<Array<T>>;
    /**
     * getDatasByRids 와 같은 역할을 하지만 데이터가 아니라, 데이터의 카운트를 구한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param rids
     * @param transaction
     * @throws CkError
     */
    static countDatasByRids_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, rids: Array<string>, transaction?: Transaction): Promise<number>;
    /**
     * 필터 조건으로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param filter
     * @param attributes
     * @param transaction
     * @param order
     * @throws CkError
     */
    static getDataOneByFilter_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<T | null>;
    /**
     * 필터 조건으로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     *
     * order 는 다음과 같이 사용 한다.
     * - [['name', 'DESC']] : name 칼럼으로 desc 정렬
     * - [['name', 'DESC'], ['id', 'ASC']] : name 칼럼으로 desc 정렬하고, 두번째로 id 로 ASC 정렬
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param filter
     * @param attributes
     * @param transaction
     * @param order
     * @param offset
     * @param limit
     * @throws CkError
     */
    static getDatasByFilter_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order, offset?: number, limit?: number): Promise<Array<T>>;
    /**
     * getDatasByFilter 와 동일한 역할을 하지만, 데이터가 아니라 데이터의 카운트를 구한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param filter
     * @param transaction
     * @throws CkError
     */
    static countDatasByFilter_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, transaction?: Transaction): Promise<number>;
    /**
     * key array 로 데이터 목록을 구한다. 가령 file_id array 로 파일 데이터를 map 형태로 구 할 때 사용 한다.
     * @param req
     * @param res
     * @param keys select 조건 key array
     * @param VCreator select 할 테이블
     * @param colName 테이블에서 select 할 칼럼명
     * @param mapKey map 으로 만들때 키가되는 칼럼명(k_칼럼값 형태로 키가 생성된다.)
     * @param cols 테이블에서 특정 칼럼만 가져오고 싶을때 지정 칼럼 목록
     * @param toCond 검색할 테이블 select 시에 추가할 조건
     * @param transaction transaction object
     */
    static getDataMapByKeyArray_AOP<T, V>(req: express.Request, res: express.Response, keys: Array<T>, VCreator: {
        new (): V;
    }, colName: string, mapKey: string, cols?: Array<string>, toCond?: {
        [key: string]: any;
    }, transaction?: Transaction): Promise<{
        [key: string]: any;
    }>;
    /**
     * list_order 순서를 변경 한다.
     * raw query 는 함수 내에서 추가 조건이 들어가고, 이에 따라 replacement 도 추가 조건이 들어 간다.
     * list_order 변경 관련 값은 함수 내부에서 처리 되고, 파라미터로 주는 raw query 는 데이터 검색 조건만 들어 가도록 해야 한다.
     * 움직이는 위치에 따라 list_order + 1, list_order -1 을 해야 하기 때문에 파라미터로 넘겨주는 rawQuery 내에 <list_order> 스트링으로
     * 값을 설정 한다.
     *
     * 순수하게 list_order 만 변경 하고, list_order 변경 가능한지 데이터 비교 하지 않는다는 것이 changeListOrderV2 와 다른점 이다.
     *
     * 사용 예제는 한국문화교육의 ContentCategoryAdmRouter.ts 의 카테고리 위치 변경을 참고 하자.
     * 또는 ck-gw-core 의 MenuAdmRouter.putLorder 를 참고 한다.
     *
     * @param req express request object
     * @param res express response object
     * @param rdb sequelize object
     * @param TCreator 테이블
     * @param rawQuery 부분 raw 쿼리
     * @param replacements raw 쿼리에서 replace 값
     * @param srcId 순서를 변경할 src 의 id
     * @param srcListOrder 순서를 변경할 src 의 list_order
     * @param destListOrder 순서를 변경할 dest 의 list_order
     * @param transaction transaction
     * @throws CkError
     */
    static changeListOrder_AOP<T>(req: express.Request, res: express.Response, rdb: SequelizeTS, TCreator: {
        new (): T;
    }, rawQuery: string, replacements: {
        [key: string]: any;
    } | undefined, srcId: number, srcListOrder: number, destListOrder: number, transaction: Transaction): Promise<number>;
    /**
     * list_order 순서를 변경 한다. router 에서 순서 변경하는 것을 함수 한번에 처리 할 수 있도록 묶어 둔 것이다.
     * 간단한 순서 변경에 사용 하고, 추가 제한 조건이 있으면 본 함수를 사용 하지 않도록 한다.
     * 에러 나면 string 을 리턴 하는데, 이런 경우는 호출 하는 곳에서 transaction rollback 처리 해 주어야 한다.
     *
     * changeListOrder 와 다른점은 함수 내부에서 순서를 변경 하려는 데이터를 찾아, 데이터 존재 여부 및 list_order 가 동일한지 비교 한다.
     *
     * 사용 예제는 한국문화교육의 VRCAdminRouter.ts 의 키워드 위치 변경을 참고 하자.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req express request object
     * @param res express response object
     * @param rdb sequelize object
     * @param TCreator 테이블
     * @param srcRid 순서를 변경할 src 의 rid
     * @param destRid 순서를 변경할 dest 의 rid
     * @param rawQueryPart 부분 raw 쿼리
     * @param replacements raw 쿼리에서 replace 값
     * @param transaction transaction
     * @throws CkError
     */
    static changeListOrderV2_AOP<T>(req: express.Request, res: express.Response, rdb: SequelizeTS, TCreator: {
        new (): T;
    }, srcRid: string, destRid: string, rawQueryPart: string, replacements: {
        [key: string]: any;
    } | undefined, transaction: Transaction): Promise<number>;
    /**
     * 새로운 데이터를 생성 하고, list_order 를 생성된 srl 값과 동일하게 설정 한다.
     * list_order 가 있는 대부분의 데이터 생성시 사용 하면 된다.
     *
     * string 이 리턴 되면, 에러 처리 해야 한다. 에러가 발생 되면 호출 하는 곳에서 transaction rollback 처리 해야 한다.
     * 한국 문화 교육의 VRCKeyword 생성을 참고 한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreator
     * @param newData
     * @param transaction
     * @throws CkError
     */
    static createDataWithListOrder_AOP<T>(req: express.Request, res: express.Response, TCreator: {
        new (): T;
    }, newData: T, transaction: Transaction): Promise<T>;
    /**
     * 데이터를 저장 한다. 정상으로 실행 되면 저장된 데이터를 리턴 하고, 에러이면 에러 코드(string)을 리턴 한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param newData
     * @param transaction
     * @throws CkError
     */
    static createData_AOP<T>(req: express.Request, res: express.Response, newData: T, transaction?: Transaction): Promise<T>;
    /**
     * 데이터를 업데이트 한다. number 를 리턴하면, 업데이트 데이터 카운트 이며, 스트링이면 에러 코드 이다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreate
     * @param modData
     * @param where
     * @param transaction
     * @throws CkError
     */
    static updateData_AOP<T>(req: express.Request, res: express.Response, TCreate: {
        new (): T;
    }, modData: any, where: any, transaction?: Transaction): Promise<number>;
    /**
     * 데이터를 삭제 한다. number 를 리턴하면, 삭제된 데이터 카운트 이며, 스트링 이면 에러 코드 이다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreate
     * @param where
     * @param transaction
     * @throws CkError
     */
    static deleteData_AOP<T>(req: express.Request, res: express.Response, TCreate: {
        new (): T;
    }, where: any, transaction?: Transaction): Promise<number>;
    /**
     * 칼럼 값을 증가 시키거나, 감소 시킨다. true 를 리턴하면 처리 성공이고, 스트링 이면 에러 코드 이다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param TCreate
     * @param where 쿼리 검색 조건
     * @param cols increase, decrease 시킬 칼럼명. 일반적으로 칼럼 하나만 사용 될 것이지만, 여러개 칼럼을 처리 할 수 있다.
     * @param by increase, decrease 시키는 양. 1 이면 1씩 증가하고, -1 이면 1씩 감소 한다.
     * @param transaction
     * @throws CkError
     */
    static incrementData_AOP<T>(req: express.Request, res: express.Response, TCreate: {
        new (): T;
    }, where: any, cols: Array<string>, by: number, transaction?: Transaction): Promise<void>;
    /**
     * raw query 를 사용하여 데이터를 select 한다.
     *
     * 에러 발생시 에러를 throw 한다.
     * @param req
     * @param res
     * @param rawQuery
     * @param rdb
     * @param replacements
     * @param transaction
     * @throws CkError
     */
    static getDataByRawQuery_AOP(req: express.Request, res: express.Response, rawQuery: string, rdb: SequelizeTS | number, replacements?: {
        [col: string]: any;
    }, transaction?: Transaction): Promise<Array<{
        [col_name: string]: any;
    }>>;
    /**
     * table select 시에 칼럼명을 지정해서 select 할때, 필수로 존재해야 하는 칼럼이 있을 경우, 칼럼명을 추가 한다.
     * undefined 로 select 하면 모든 칼럼을 select 하기 때문에, 원본값(attributes) 가 undefined 이면 undefined 가 리턴 된다.
     * @param columnNames
     * @param attributes
     */
    static addMandatoryColumns(columnNames: Array<string>, attributes?: Array<string>): Array<string> | undefined;
}
export default DatabaseWebService;
